@@ -1,7 +1,7 @@
 """Configures and builds c/c++ source, given target configurations."""
 
 import os
-import multiprocessing
+import subprocess
 
 #sudo easy_install termcolor
 import termcolor
@@ -126,7 +126,23 @@ class Builder:
         return True
 
     @staticmethod
-    def ensure_dependency_exists(path):
+    def build_dependency(path, target_name):
+        """build dependency at location
+
+        Args:
+            path: path to SConstruct file
+            target_name: target to build
+        """
+
+        parent_path = os.getcwd()
+        os.chdir(path)
+        build_failed = subprocess.call(['scons', target_name])
+        os.chdir(parent_path)
+        error = 'error: could not build %s:%s' % (path, target_name) 
+        bbricks.output.assert_or_die(build_failed == 0, error)
+
+    @staticmethod
+    def ensure_file_exists(path):
         """check file exists, abort execution otherwise
 
         Args:
@@ -138,26 +154,46 @@ class Builder:
         error = "error: %s is not installed on your machine" % path
         bbricks.output.assert_or_die(exists, error)
 
-
     @staticmethod
-    def __check_installed_deps(target):
-        """Check all dependencies exist for target
+    def resolve_dependency(path, target_name):
+        """resolve external build ependencies
 
         Args:
-            target: target dict to check
+            path: can be either file or directory. files are checked for existance. directories invoke scons on it.
+            target_name: target name to invoke in dependency
+        """
+
+        if os.path.isfile(path):
+            Builder.ensure_file_exists(path)
+            return
+
+        if os.path.isdir(path):
+            Builder.build_dependency(path, target_name)
+            return
+
+        error = "error: dependency %s is neither a file nor a directory" % path
+        bbricks.output.assert_or_die(exists, error)
+
+    @staticmethod
+    def __resolve_dependencies(target):
+        """Build or check external dependencies
+
+        Args:
+            target: target dict to resolve dependencies for
         """
         
         if not target.has_key('depends'):
             return
 
         deps = target['depends']
+        target_name = target['name']
         if isinstance(deps, str):
-            Builder.ensure_dependency_exists(deps)
+            Builder.resolve_dependency(deps, target_name)
             return
 
         if isinstance(deps, list):
             for dep in deps:
-                Builder.ensure_dependency_exists(dep)
+                Builder.resolve_dependency(dep, target_name)
             return
 
         # FIXME move to schema validation
@@ -185,7 +221,7 @@ class Builder:
     def __add_multiple_job_support():
         """Configure scons to start as many jobs as cores"""
 
-        num_cpu = int(multiprocessing.cpu_count())
+        num_cpu = int(os.environ.get('NUM_CPU', 2))
         SCons.Script.SetOption('num_jobs', num_cpu)
 
     @staticmethod
@@ -304,7 +340,7 @@ class Builder:
 
         target = self.get_target(target_names[0])
 
-        self.__check_installed_deps(target)
+        self.__resolve_dependencies(target)
         self.__setup_toolchain(target)
         self.__add_build_options(target)
         self.__add_multiple_job_support()
